/*
Copyright (c) 2019 - 2025 Advanced Micro Devices, Inc. All rights reserved.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "pipeline/node.h"

Node::~Node() {
    if (_node) vxReleaseNode(&_node);
    _node = nullptr;
}

void Node::release() {
    _prev.clear();
    _next.clear();
}

void Node::create(std::shared_ptr<Graph> graph) {
    if (_outputs.empty() || _inputs.empty())
        THROW("Uninitialized input/output images to the node")

    _graph = graph;
    create_node();
}

void Node::update_parameters() {
    update_node();
}

void Node::add_next(const std::shared_ptr<Node> &node) {
    // Set graph ID to the Node, to denote the graph to which it belongs to
    if (node->get_graph_id() < 0) node->set_graph_id(_graph_id);
    _next.emplace_back(node);
}

void Node::add_previous(const std::shared_ptr<Node> &node) {
    for (auto& prev_node : _prev) {
        if (prev_node->get_graph_id() != node->get_graph_id())
            THROW("The nodes are interdependent between 2 graphs")
    }
    _prev.emplace_back(node);
}
